// -*- C++ -*-
/*!
 * @file  rtc_2dcdp.h
 * @brief Image processing of the 2DCDP
 * @date  $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */

#ifndef RTC_2DCDP_H
#define RTC_2DCDP_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

//add to code
#include <iostream>
#include <string>
#include <stdio.h>
#include "windows.h"

// OpenCV
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "RTC_2DCDPSystemControlSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "ImageLoaderControlStub.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

// add to code
#include "local_basic_type.h"
#include "2dcdp_system.h"

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

//add to code
/*=================================================================================================
  Macro definition
 ================================================================================================*/
//#define DEBUG_SW
//#define GET_IMAGE_FROM_IMAGEFILE

//-------------------------------------------------------------------------------------------------
#define _2DCDP_STAGE_INIT              (0)
#define _2DCDP_STAGE_MIN               (_2DCDP_STAGE_INIT)
#define _2DCDP_STAGE1                  (1)
#define _2DCDP_STAGE2                  (2)
#define _2DCDP_STAGE3                  (3)
#define _2DCDP_STAGE4                  (4)
#define _2DCDP_STAGE5                  (5)
#define _2DCDP_STAGE_MAX               (_2DCDP_STAGE5+1)

//-------------------------------------------------------------------------------------------------
#define ERROR_RETRY_INIT               (0)
#define ERROR_RETRY_JUGE               (0)
#define ERROR_RETRY_1                  (1)
#define ERROR_RETRY_2                  (2)
#define ERROR_RETRY_OK                 (0)
#define ERROR_RETRY_NG                 (1)

//-------------------------------------------------------------------------------------------------
#define OPT_DATA_INDEX_0               (0)  //m_2dcdp_opt.data[0]; Parameter Set or Default Parameter Set
#define OPT_DATA_INDEX_1               (1)  //m_2dcdp_opt.data[1]; 出力ファイル識別タグ指定
#define OPT_DATA_INDEX_2               (2)  //m_2dcdp_opt.data[2]; 取り出し数指定
#define OPT_DATA_INDEX_3               (3)  //m_2dcdp_opt.data[3]; 上がり連続縮小制限指定
#define OPT_DATA_INDEX_4               (4)  //m_2dcdp_opt.data[4]; 下がり連続縮小制限指定
#define OPT_DATA_INDEX_5               (5)  //m_2dcdp_opt.data[5]; 背景色(出力)指定 : red
#define OPT_DATA_INDEX_6               (6)  //m_2dcdp_opt.data[6]; 背景色(出力)指定 : green
#define OPT_DATA_INDEX_7               (7)  //m_2dcdp_opt.data[7]; 背景色(出力)指定 : blue
#define OPT_DATA_INDEX_8               (8)  //m_2dcdp_opt.data[8]; 背景色(参照)指定 : red
#define OPT_DATA_INDEX_9               (9)  //m_2dcdp_opt.data[9]; 背景色(参照)指定 : green
#define OPT_DATA_INDEX_10              (10) //m_2dcdp_opt.data[10];背景色(参照)指定 : blue

//-------------------------------------------------------------------------------------------------
#define DEFAULT_EXTNUM                 (1)

//-------------------------------------------------------------------------------------------------
#define FORMAT_COLOR24                 (24)

//-------------------------------------------------------------------------------------------------
#define CONFIG_ENABLE_SW               (0)
#define CONFIG_DISABLE_SW              (1)

//-------------------------------------------------------------------------------------------------
#define IMAGES_LENGTH_6                (6)
#define OPT_DATA_LENGTH_11             (11)

//-------------------------------------------------------------------------------------------------
#define BUFFER_SIZE_1024               (1024)

//-------------------------------------------------------------------------------------------------
#define OPTION_DATA_EXIST              (0)
#define OPTION_DATA_EMPTY              (1)

//-------------------------------------------------------------------------------------------------
#define _2DCDP_IMAGE_EXIST             (0)
#define _2DCDP_IMAGE_EMPTY             (1)

#define _2DCDP_IMAGE_RESIZE_ERR        (2)
#define _2DCDP_IMAGE_COLOR_ERR         (3)
#define _2DCDP_IMAGE_SIZE_ERR          (4)

//-------------------------------------------------------------------------------------------------
#define INP_IMAGE_RES_CHK_BOTH         (0)
#define INP_IMAGE_RES_CHK_PORT1        (1)
#define INP_IMAGE_RES_CHK_PORT2        (2)

//message definition
#define _2DCDP_ON_INITIALIZE           ("onInitialize execution")
#define _2DCDP_ON_ACTIVATED            ("onActivated execution")
#define _2DCDP_ON_DEACTIVATED          ("onDeactivated execution")
#define _2DCDP_ON_ABORTING             ("onAborting execution")
#define _2DCDP_ON_ERROR                ("onError execution")
#define _2DCDP_ON_RESET                ("onReset execution")

#define _2DCDP_SUCCESS                 ("2DCDP process success")
#define _2DCDP_FAILURE                 ("2DCDP process failure")

#define _2DCDP_STAGE1_MESSAGE          ("STAGE1: Checks of input parameters")
#define _2DCDP_STAGE2_MESSAGE          ("STAGE2: Checks of the input image data")
#define _2DCDP_STAGE3_MESSAGE          ("STAGE3: Execution of 2DCDP core logic")
#define _2DCDP_STAGE4_MESSAGE          ("STAGE4: Image data storage generated by 2dcdp process")
#define _2DCDP_STAGE5_MESSAGE          ("STAGE5: Creating multi-camera image data")
#define _2DCDP_STAGE_ERROR             ("ERROR: Wrong value of the stage variable")

#define _2DCDP_ERROR_IMAGE_COLOR       ("ERROR: This image file is not implemented in this program -> ")
#define _2DCDP_ERROR_IMAGE_SIZE        ("ERROR: This image file is outside the size target -> ")
#define _2DCDP_ERROR_IMAGE_WRITE       ("ERROR: Image write failure: ")
#define _2DCDP_ERROR_IMAGE_EMPTY       ("ERROR: Empty error of image file: ")
//-------------------------------------------------------------------------------------------------
#define _2DCDP_INPUT_FILE              (0)
#define _2DCDP_REFERENCE_FILE          (1)
#define _2DCDP_LABEL_FILE              (2)
//-------------------------------------------------------------------------------------------------
#define RESULT_2DCDP_OK                (0)
#define RESULT_2DCDP_NG                (1)
#define RESULT_2DCDP_CONTINUE          (2)
//-------------------------------------------------------------------------------------------------
#define RTC_2DCDP_IMG_INP              (0)
#define RTC_2DCDP_IMG_REF              (1)
#define RTC_2DCDP_IMG_LBL              (2)

#define RTC_2DCDP_IMG_INO              (3)
#define RTC_2DCDP_IMG_PRO              (4)
#define RTC_2DCDP_IMG_RFO              (5)
#define RTC_2DCDP_IMG_LBO              (6)
#define RTC_2DCDP_IMG_DIF              (7)

#define RTC_2DCDP_TXT_CDP              (8)
//-------------------------------------------------------------------------------------------------
#define RTC_CAMERA_IMAGE_INP           (0)
#define RTC_CAMERA_IMAGE_MIN           (RTC_CAMERA_IMAGE_INP)
#define RTC_CAMERA_IMAGE_REF           (1)
#define RTC_CAMERA_IMAGE_INO           (2)
#define RTC_CAMERA_IMAGE_PRO           (3)
#define RTC_CAMERA_IMAGE_RFO           (4)
#define RTC_CAMERA_IMAGE_DIF           (5)
#define RTC_CAMERA_IMAGE_MAX           (RTC_CAMERA_IMAGE_DIF+1)
#define RTC_CAMERA_IMAGE_LBL           (6) //Not yet determined
#define RTC_CAMERA_IMAGE_LBO           (7) //Not yet determined
//-------------------------------------------------------------------------------------------------
#define XIMG_MIN                       (0)
#define YIMG_MIN                       (0)
#define XIMG_MAX                       (640)
#define YIMG_MAX                       (480)

#define XIMG_120                       (120)
#define YIMG_90                        (90)

#define XIMG_240                       (240)
#define YIMG_180                       (180)

#define XIMG_320                       (320)
#define YIMG_240                       (240)
//-------------------------------------------------------------------------------------------------
#define SHR_UP_MIN                     (1)
#define SHR_UP_MAX                     (2)

#define SHR_DN_MIN                     (1)
#define SHR_DN_MAX                     (2)
//-------------------------------------------------------------------------------------------------
#define RTC_EC_TIME_0100MS             (1000/100)
#define RTC_EC_TIME_0500MS             (1000/500)
#define RTC_EC_TIME_1000MS             (1000/1000)
#define RTC_EC_TIME_2500MS             (1000/2500)
#define RTC_EC_TIME_5000MS             (1000/5000)
//-------------------------------------------------------------------------------------------------
#define S_ARRAY_256                    (256)
//-------------------------------------------------------------------------------------------------
#define IMAGE_NO_WRITE                 (0)
#define IMAGE_OK_WRITE                 (1)
//-------------------------------------------------------------------------------------------------
#define STAGE2_GUARD_JUDGE             (0)
#define STAGE2_GUARD_COUNTER           (600)
//-------------------------------------------------------------------------------------------------

/*!
 * @class rtc_2dcdp
 * @brief Image processing of the 2DCDP
 *
 * カメライメージ型の入力画像データと参照画像データを使用して、二次
 * 元連続動的計画法(２ＤＣＤＰ)のアルゴリズムを用いた非線形最適マッ
 * チングの処理を行う
 *
 * 入力：カメライメージ型の入力画像データと参照画像データを入力する
 * 出力：カメライメージ型の入力画像データと参照画像データ／入力復元
 * 画像データ／入力投影画像データ／参照投影画像データ／距離差分画像
 * データを出力する。また、入力参照対応用ファイルのフルパス情報をT
 * imedString型で出力する
 *
 * 機能仕様書_2dcdp.docx参照
 *
 * なし
 *
 */
class rtc_2dcdp
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  rtc_2dcdp(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~rtc_2dcdp();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * Initialization of the 2dcdp RTC
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post ｰ
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * Activate the 2dcdp RTC
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * Deactivate the 2dcdp RTC
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * Periodic task of the 2dcdp RTC
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   * onAborting the 2dcdp RTC
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   * onError the 2dcdp RTC
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   * Reset initialization of the 2dcdp RTC
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 本Parameterで入力画像データの出力を制御する
   * - Name: Image_inp_switch image_inp_sw
   * - DefaultValue: 0
   * - Unit: radio(0:enable,1:disable)
   * - Range: 0 or 1(0:enable,1:disable)
   * - Constraint: 指定なし（none specified）
   */
  int m_image_inp_sw;
  /*!
   * 本Parameterで参照画像データの出力を制御する
   * - Name: Image_ref_switch image_ref_sw
   * - DefaultValue: 0
   * - Unit: radio(0:enable,1:disable)
   * - Range: 0 or 1(0:enable,1:disable)
   * - Constraint: 指定なし（none specified）
   */
  int m_image_ref_sw;
  /*!
   * 本Parameterで入力復元用画像データの出力を制御する
   * - Name: Image_op_switch image_op_sw
   * - DefaultValue: 0
   * - Unit: radio(0:enable,1:disable)
   * - Range: 0 or 1(0:enable,1:disable)
   * - Constraint: 指定なし（none specified）
   */
  int m_image_op_sw;
  /*!
   * 本Parameterで入力投影用画像データの出力を制御する
   * - Name: Image_oi_switch image_oi_sw
   * - DefaultValue: 0
   * - Unit: radio(0:enable,1:disable)
   * - Range: 0 or 1(0:enable,1:disable)
   * - Constraint: 0 or 1(0:enable,1:disable)
   */
  int m_image_oi_sw;
  /*!
   * 本Parameterで参照投影用画像データの出力を制御する
   * - Name: Image_or_switch image_or_sw
   * - DefaultValue: 0
   * - Unit: radio(0:enable,1:disable)
   * - Range: 0 or 1(0:enable,1:disable)
   * - Constraint: 指定なし（none specified）
   */
  int m_image_or_sw;
  /*!
   * 本Parameterで距離差分用画像データの出力を制御する
   * - Name: Image_dif_switch image_dif_sw
   * - DefaultValue: 0
   * - Unit: radio(0:enable,1:disable)
   * - Range: 0 or 1(0:enable,1:disable)
   * - Constraint: 指定なし（none specified）
   */
  int m_image_dif_sw;
  /*!
   * 本Parameterで入力参照対応用ファイルパス情報の出力を制御する
   * - Name: File_cdp_switch file_cdp_sw
   * - DefaultValue: 0
   * - Unit: radio(0:enable,1:disable)
   * - Range: 0 or 1(0:enable,1:disable)
   * - Constraint: 指定なし（none specified）
   */
  int m_file_cdp_sw;
  /*!
   * 本Parameterで画像ファイル書き込み有り無しを制御する
   * - Name: ImageFile_write_switch image_write_sw
   * - DefaultValue: 0
   * - Unit: radio(0:not write,1:write)
   * - Range: 0 or 1
   * - Constraint: 指定なし（none specified）
   */
  int m_image_write_sw;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::CameraImage m_image1;
  /*!
   * 入力画像データを取得する
   * - Type: RTC::CameraImage
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 2.5秒周期
   * - Operation Cycle: 2.5秒周期
   */
  InPort<RTC::CameraImage> m_image1In;
  RTC::CameraImage m_image2;
  /*!
   * 参照画像データを取得する
   * - Type: RTC::CameraImage
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 2.5秒周期
   * - Operation Cycle: 2.5秒周期
   */
  InPort<RTC::CameraImage> m_image2In;
  RTC::TimedStringSeq m_2dcdp_opt;
  /*!
   * 2DCDPセッティングデータを取得する
   * - Type: RTC::TimedStringSeq
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 2.5秒周期
   * - Operation Cycle: 2.5秒周期
   */
  InPort<RTC::TimedStringSeq> m_2dcdp_optIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::MultiCameraImages m_Image_2dcdp;
  /*!
   * 2DCDP処理された画像データをMultiCameraImages型のデータで出力す
   * る
   * - Type: RTC::MultiCameraImages
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 2.5秒周期
   * - Operation Cycle: 2.5秒周期
   */
  OutPort<RTC::MultiCameraImages> m_Image_2dcdpOut;
  RTC::TimedString m_Text_2dcdp;
  /*!
   * 入力参照対応用ファイルのフルパス情報をTimedString型のデータで
   * 出力する
   * - Type: RTC::TimedString
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 2.5秒周期
   * - Operation Cycle: 2.5秒周期
   */
  OutPort<RTC::TimedString> m_Text_2dcdpOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   * 画像ファイル読み込み動作を要求する
   * Interface: 画像ファイル読み込み動作要求のI/Fを使用する
   */
  RTC::CorbaPort m_Control2dcdpServPort;
  /*!
   * 2dcdp RTCのシステム制御
   * Interface: リセット制御のI/Fを提供する
   */
  RTC::CorbaPort m_RTC_2DCDPSystemContServPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   * 2dcdp RTCのリセット制御
   * - Argument:      Resetのトリガ
   *                  　RESET_TRIG_ON：Reset On
   *                  　RESET_TRIG_OFF：Reset Off
   * - Return Value:  なし
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: ｰ
   */
  RTC_2DCDPSystemContResetSVC_impl m_RTC_2DCDPSystemContResetProv;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   * 画像ファイル読み込み開始/停止を要求する
   * - Argument:      画像ファイル読み込み動作のトリガ
   *                  　IMLD_TRIG_START：開始
   *                  　IMLD_TRIG_STOP：停止
   * - Return Value:  なし
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  RTC::CorbaConsumer<ImageLoaderControlServ::ImageLoaderTrigger> m_Image2dcdpTriggerCons;
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

  // add to code
  void nvd2dcdpOnInitialize( void );
  void nvd2dcdpOnActivated( void );
  void nvd2dcdpOnDeactivated( void );
  void nvd2dcdpOnExecute( void );
  void nvd2dcdpOnAborting( void );
  void nvd2dcdpOnError( void );
  void nvd2dcdpOnReset( void );
  //---------------------------------------------------------
  uWord_Type uW2dcdpOnExecuteStage1( void );
  uWord_Type uW2dcdpOnExecuteStage2( void );
  uWord_Type uW2dcdpOnExecuteStage3( void );
  uWord_Type uW2dcdpOnExecuteStage4( void );
  uWord_Type uW2dcdpOnExecuteStage5( void );
  //---------------------------------------------------------
  void nvdSet2dcdpStageControl( uByte_Type );
  //---------------------------------------------------------
  uWord_Type uWJugdeErrorRetry( void );
  //---------------------------------------------------------
  uWord_Type uWCheck2dcdpParameter( void );
  void nvdSetDefaultParameter( void );
  void nvdSetFileName( Char_Type *, Char_Type *, uWord_Type );
  Char_Type *GetFileName_p( uWord_Type );
  //---------------------------------------------------------
  uWord_Type uWCheck2dcdpInputImage( void );
  uWord_Type uWCheck2dcdpInputImagePort1( void );
  uWord_Type uWCheck2dcdpInputImagePort2( void );
  uWord_Type uWResizeInputImage( RTC::CameraImage *, Char_Type *, uByte_Type );
  uWord_Type uWWriteInputImage( Char_Type *, cv::Mat );
  uWord_Type uWCheckErrorInputImage( uWord_Type, uWord_Type, uByte_Type );
  uWord_Type uWGuardInputPath( void );
  //---------------------------------------------------------
  uWord_Type uWWriteImage( Char_Type *, const IplImage * );
  //---------------------------------------------------------
  uWord_Type uWCreationCameraImages( void );
  uWord_Type uWSetCameraImages_Inp( void );
  uWord_Type uWSetCameraImages_Ref( void );
  uWord_Type uWSetCameraImages_Pro( void );
  uWord_Type uWSetCameraImages_Ino( void );
  uWord_Type uWSetCameraImages_Rfo( void );
  uWord_Type uWSetCameraImages_Def( void );
  //---------------------------------------------------------
  uWord_Type uWSetCameraImages( Char_Type *, uWord_Type );
  uWord_Type uWSetCameraImagesbyIplImage( uWord_Type auWKind );
  uWord_Type uWSetCameraImagesData( cv::Mat, uWord_Type );
  void nvdSetDefaultCameraImages( uWord_Type );
  //---------------------------------------------------------
  uWord_Type uWCreationInputRefFile( void );
  //---------------------------------------------------------
  Bool_Type bCheckCoreLogic( void );
  void nvdDeleteCore2dcdp( void );
  //---------------------------------------------------------
  void nvdExecutionCycleChange( Double_Type );
  //---------------------------------------------------------
  void nvdCycleTimeMeasurement( void );
  //---------------------------------------------------------
  IplImage IplImage_buf;

  uByte_Type m_uB2dcdpStageCtrl;
  uByte_Type m_uBResultPort1;
  uByte_Type m_uBResultPort2;
  uByte_Type m_uBRetryCounter;
  uByte_Type m_uBErrorCounter;
  uWord_Type m_uWGuardCounter;

  Char_Type inpfile[S_ARRAY_256];
  Char_Type reffile[S_ARRAY_256];
  Char_Type lblfile[S_ARRAY_256];
  Char_Type inpout[S_ARRAY_256];
  Char_Type projout[S_ARRAY_256];
  Char_Type refout[S_ARRAY_256];
  Char_Type lblout[S_ARRAY_256];
  Char_Type diffdist[S_ARRAY_256];
  Char_Type DummyName[64];

};


extern "C"
{
  DLL_EXPORT void rtc_2dcdpInit(RTC::Manager* manager);
};

#endif // RTC_2DCDP_H
