// -*- C++ -*-
/*!
 * @file  rtc_2dcdp.cpp
 * @brief Image processing of the 2DCDP
 * @date $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */

#include "rtc_2dcdp.h"
#include "2DCDP4.h"


// Module specification
// <rtc-template block="module_spec">
static const char* rtc_2dcdp_spec[] =
  {
    "implementation_id", "rtc_2dcdp",
    "type_name",         "rtc_2dcdp",
    "description",       "Image processing of the 2DCDP",
    "version",           "1.0",
    "vendor",            "EAC",
    "category",          "Image",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Image_inp_switch", "0",
    "conf.default.Image_ref_switch", "0",
    "conf.default.Image_op_switch", "0",
    "conf.default.Image_oi_switch", "0",
    "conf.default.Image_or_switch", "0",
    "conf.default.Image_dif_switch", "0",
    "conf.default.File_cdp_switch", "0",
    "conf.default.ImageFile_write_switch", "0",
    // Widget
    "conf.__widget__.Image_inp_switch", "radio",
    "conf.__widget__.Image_ref_switch", "radio",
    "conf.__widget__.Image_op_switch", "radio",
    "conf.__widget__.Image_oi_switch", "radio",
    "conf.__widget__.Image_or_switch", "radio",
    "conf.__widget__.Image_dif_switch", "radio",
    "conf.__widget__.File_cdp_switch", "radio",
    "conf.__widget__.ImageFile_write_switch", "radio",
    // Constraints
    "conf.__constraints__.Image_inp_switch", "(0,1)",
    "conf.__constraints__.Image_ref_switch", "(0,1)",
    "conf.__constraints__.Image_op_switch", "(0,1)",
    "conf.__constraints__.Image_oi_switch", "(0,1)",
    "conf.__constraints__.Image_or_switch", "(0,1)",
    "conf.__constraints__.Image_dif_switch", "(0,1)",
    "conf.__constraints__.File_cdp_switch", "(0,1)",
    "conf.__constraints__.ImageFile_write_switch", "(0,1)",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
rtc_2dcdp::rtc_2dcdp(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_image1In("ImageData_In1", m_image1),
    m_image2In("ImageData_In2", m_image2),
    m_2dcdp_optIn("Setting_2dcdp", m_2dcdp_opt),
    m_Image_2dcdpOut("ImageData_2dcdp_Out", m_Image_2dcdp),
    m_Text_2dcdpOut("TextData_2dcdp_Out", m_Text_2dcdp),
    m_Control2dcdpServPort("Control2dcdpServ"),
    m_RTC_2DCDPSystemContServPort("RTC_2DCDPSystemContServ")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
rtc_2dcdp::~rtc_2dcdp()
{
}


//add to code
/*=================================================================================================
  Variables definition
 ================================================================================================*/
CDP* m_core2dcdp;

#ifdef DEBUG_SW
uWord_Type rtc_start;
uWord_Type rtc_end;
uByte_Type m_cycle_time = 0;
#endif
//-------------------------------------------------------------------------------------------------

/*!
 * Initialization of the 2dcdp RTC
 */
RTC::ReturnCode_t rtc_2dcdp::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("ImageData_In1", m_image1In);
  addInPort("ImageData_In2", m_image2In);
  addInPort("Setting_2dcdp", m_2dcdp_optIn);
  
  // Set OutPort buffer
  addOutPort("ImageData_2dcdp_Out", m_Image_2dcdpOut);
  addOutPort("TextData_2dcdp_Out", m_Text_2dcdpOut);
  
  // Set service provider to Ports
  m_RTC_2DCDPSystemContServPort.registerProvider("RTC_2DCDPSystemContReset", "RTC_2DCDPSystemContServ::RTC_2DCDPSystemContReset", m_RTC_2DCDPSystemContResetProv);
  
  // Set service consumers to Ports
  m_Control2dcdpServPort.registerConsumer("ImageLoaderTrigger", "ImageLoaderControlServ::ImageLoaderTrigger", m_Image2dcdpTriggerCons);
  
  // Set CORBA Service Ports
  addPort(m_Control2dcdpServPort);
  addPort(m_RTC_2DCDPSystemContServPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Image_inp_switch", m_image_inp_sw, "0");
  bindParameter("Image_ref_switch", m_image_ref_sw, "0");
  bindParameter("Image_op_switch", m_image_op_sw, "0");
  bindParameter("Image_oi_switch", m_image_oi_sw, "0");
  bindParameter("Image_or_switch", m_image_or_sw, "0");
  bindParameter("Image_dif_switch", m_image_dif_sw, "0");
  bindParameter("File_cdp_switch", m_file_cdp_sw, "0");
  bindParameter("ImageFile_write_switch", m_image_write_sw, "0");
  // </rtc-template>
  
  // add to code
  nvd2dcdpOnInitialize();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t rtc_2dcdp::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t rtc_2dcdp::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t rtc_2dcdp::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Activate the 2dcdp RTC
 */

RTC::ReturnCode_t rtc_2dcdp::onActivated(RTC::UniqueId ec_id)
{
  // add to code
  nvd2dcdpOnActivated();

  return RTC::RTC_OK;
}

/*!
 * Deactivate the 2dcdp RTC
 */

RTC::ReturnCode_t rtc_2dcdp::onDeactivated(RTC::UniqueId ec_id)
{
  // add to code
  nvd2dcdpOnDeactivated();

  return RTC::RTC_OK;
}

/*!
 * Periodic task of the 2dcdp RTC
 */

RTC::ReturnCode_t rtc_2dcdp::onExecute(RTC::UniqueId ec_id)
{
#ifdef DEBUG_SW
  nvdCycleTimeMeasurement();
#endif

  // add to code
  nvd2dcdpOnExecute();

  return RTC::RTC_OK;
}

/*!
 * onAborting the 2dcdp RTC
 */

RTC::ReturnCode_t rtc_2dcdp::onAborting(RTC::UniqueId ec_id)
{
  // add to code
  nvd2dcdpOnAborting();

  return RTC::RTC_OK;
}

/*!
 * onError the 2dcdp RTC
 */

RTC::ReturnCode_t rtc_2dcdp::onError(RTC::UniqueId ec_id)
{
  // add to code start
  nvd2dcdpOnError();

  uWord_Type uWReqReset = m_RTC_2DCDPSystemContResetProv.Get2DCDPSystemContReset();

  if( uWReqReset == RESET_TRIG_ON )
  {
	  get_context(ec_id)->reset_component( getObjRef() );

	  //Initializes the reset request
	  m_RTC_2DCDPSystemContResetProv.Req2DCDPSystemContReset( RESET_TRIG_OFF );

  }
  // add to code end

  return RTC::RTC_OK;
}

/*!
 * Reset initialization of the 2dcdp RTC
 */

RTC::ReturnCode_t rtc_2dcdp::onReset(RTC::UniqueId ec_id)
{
  // add to code start
  nvd2dcdpOnReset();

#if 0 //Go to activate
	get_context(ec_id)->activate_component( getObjRef() );
#endif
  // add to code end

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t rtc_2dcdp::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t rtc_2dcdp::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


// add to code
/***************************************************************************************************
 *  Function Name   ：nvd2dcdpOnExecute
 *  Description     ：This function execute the main process of 2DCDP.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
void rtc_2dcdp::nvd2dcdpOnExecute( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  switch ( m_uB2dcdpStageCtrl )
  {
    case _2DCDP_STAGE1:
      std::cout << _2DCDP_STAGE1_MESSAGE << std::endl;

      auWReturn = uW2dcdpOnExecuteStage1();
      if( auWReturn == RESULT_2DCDP_OK )
      {
        nvdSet2dcdpStageControl( _2DCDP_STAGE2 );
        nvdExecutionCycleChange( RTC_EC_TIME_0100MS );
      }

      break;
    //---------------------------------------------------------------------------------


    //---------------------------------------------------------------------------------
    case _2DCDP_STAGE2:
      std::cout << _2DCDP_STAGE2_MESSAGE << std::endl;

      auWReturn = uW2dcdpOnExecuteStage2();
      if( auWReturn == RESULT_2DCDP_OK )
      {
        nvdSet2dcdpStageControl( _2DCDP_STAGE3 );
        nvdExecutionCycleChange( RTC_EC_TIME_5000MS );
      }
      else if( auWReturn == RESULT_2DCDP_NG )
      {
        nvdSet2dcdpStageControl( _2DCDP_STAGE1 );
        nvdExecutionCycleChange( RTC_EC_TIME_1000MS );
      }
      else
      {
        //RESULT_2DCDP_CONTINUE
      }

      break;
    //---------------------------------------------------------------------------------


    //---------------------------------------------------------------------------------
    case _2DCDP_STAGE3:
      std::cout << _2DCDP_STAGE3_MESSAGE << std::endl;

      auWReturn = uW2dcdpOnExecuteStage3();
      if( auWReturn == RESULT_2DCDP_OK )
      {
        std::cout << _2DCDP_SUCCESS << std::endl;
        nvdSet2dcdpStageControl( _2DCDP_STAGE4 );
        nvdExecutionCycleChange( RTC_EC_TIME_0100MS );
      }
      else
      {
        if( ERROR_RETRY_OK == uWJugdeErrorRetry() )
        {
          nvdSet2dcdpStageControl( _2DCDP_STAGE3 );
        }
        else
        {
          std::cout << _2DCDP_FAILURE << std::endl;
          nvdSet2dcdpStageControl( _2DCDP_STAGE1 );
          nvdExecutionCycleChange( RTC_EC_TIME_1000MS );
        }
      }

      break;
    //---------------------------------------------------------------------------------


    //---------------------------------------------------------------------------------
    case _2DCDP_STAGE4:
      std::cout << _2DCDP_STAGE4_MESSAGE << std::endl;

      auWReturn = uW2dcdpOnExecuteStage4();
      if( auWReturn == RESULT_2DCDP_OK )
      {
        nvdSet2dcdpStageControl( _2DCDP_STAGE5 );
        nvdExecutionCycleChange( RTC_EC_TIME_0100MS );
      }
      else
      {
        if( ERROR_RETRY_OK == uWJugdeErrorRetry() )
        {
          nvdSet2dcdpStageControl( _2DCDP_STAGE4 );
        }
        else
        {
          nvdSet2dcdpStageControl( _2DCDP_STAGE1 );
          nvdExecutionCycleChange( RTC_EC_TIME_1000MS );
        }
      }

      break;
    //---------------------------------------------------------------------------------


    //---------------------------------------------------------------------------------
    case _2DCDP_STAGE5:
      std::cout << _2DCDP_STAGE5_MESSAGE << std::endl;

      auWReturn = uW2dcdpOnExecuteStage5();
      if( auWReturn == RESULT_2DCDP_OK )
      {
        nvdSet2dcdpStageControl( _2DCDP_STAGE1 );
        nvdExecutionCycleChange( RTC_EC_TIME_1000MS );
      }
      else
      {
        if( ERROR_RETRY_OK == uWJugdeErrorRetry() )
        {
          nvdSet2dcdpStageControl( _2DCDP_STAGE5 );
        }
        else
        {
          nvdSet2dcdpStageControl( _2DCDP_STAGE1 );
          nvdExecutionCycleChange( RTC_EC_TIME_1000MS );
        }
      }

      break;

    default:
      std::cout << _2DCDP_STAGE_ERROR << std::endl;
      break;
  }
}

/***************************************************************************************************
 *  Function Name   ：uW2dcdpOnExecuteStage1
 *  Description     ：This function execute the main stage1 process of 2DCDP.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uW2dcdpOnExecuteStage1( void )
{
  uWord_Type auWReturn;

  auWReturn = uWCheck2dcdpParameter();
  if( auWReturn == OPTION_DATA_EXIST )
  {
    m_Image2dcdpTriggerCons->ReqImageLoaderTrigger( IMLD_TRIG_START );
    m_uBResultPort1  = _2DCDP_IMAGE_EMPTY;
    m_uBResultPort2  = _2DCDP_IMAGE_EMPTY;
    m_uWGuardCounter = STAGE2_GUARD_COUNTER;
    auWReturn = RESULT_2DCDP_OK;
  }
  else
  {
    auWReturn = RESULT_2DCDP_NG;
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uW2dcdpOnExecuteStage2
 *  Description     ：This function execute the main stage2 process of 2DCDP.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uW2dcdpOnExecuteStage2( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  auWReturn = uWCheck2dcdpInputImage();
  if( auWReturn == RESULT_2DCDP_OK )
  {
    m_uBRetryCounter = ERROR_RETRY_1;
  }
  else
  {
    if( auWReturn != RESULT_2DCDP_NG )
    {
      auWReturn = uWGuardInputPath();
    }
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uW2dcdpOnExecuteStage3
 *  Description     ：This function execute the main stage3 process of 2DCDP.
 *                  ：And,this function execute the core logic of 2DCDP.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uW2dcdpOnExecuteStage3( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( bCheckCoreLogic() )
  {
	  if( m_core2dcdp->NextCDP() == true )
    {
      m_uBRetryCounter = ERROR_RETRY_1;
      auWReturn = RESULT_2DCDP_OK;
    }
    else
    {
      auWReturn = RESULT_2DCDP_NG;
    }
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uW2dcdpOnExecuteStage4
 *  Description     ：This function execute the main stage4 process of 2DCDP.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uW2dcdpOnExecuteStage4( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( bCheckCoreLogic() )
  {
    uWord_Type auWResult_Ino = uWWriteImage(GetFileName_p(RTC_2DCDP_IMG_INO),m_core2dcdp->GetImage_o_i());
    uWord_Type auWResult_Pro = uWWriteImage(GetFileName_p(RTC_2DCDP_IMG_PRO),m_core2dcdp->GetImage_o_p());
    uWord_Type auWResult_Rfo = uWWriteImage(GetFileName_p(RTC_2DCDP_IMG_RFO),m_core2dcdp->GetImage_o_r());
    uWord_Type auWResult_Def = uWWriteImage(GetFileName_p(RTC_2DCDP_IMG_DIF),m_core2dcdp->GetImage_dif());

    if( (auWResult_Ino == RESULT_2DCDP_OK) && (auWResult_Pro == RESULT_2DCDP_OK) &&
        (auWResult_Rfo == RESULT_2DCDP_OK) && (auWResult_Def == RESULT_2DCDP_OK) )
    {
      m_uBRetryCounter = ERROR_RETRY_1;
      auWReturn = RESULT_2DCDP_OK;
    }
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uW2dcdpOnExecuteStage5
 *  Description     ：This function execute the main stage5 process of 2DCDP.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uW2dcdpOnExecuteStage5( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;
  uWord_Type auWRetCameraImages = RESULT_2DCDP_NG;
  uWord_Type auWRetInputRefFile = RESULT_2DCDP_NG;

  auWRetCameraImages = uWCreationCameraImages();
  auWRetInputRefFile = uWCreationInputRefFile();
  if( (auWRetCameraImages == RESULT_2DCDP_OK) && (auWRetInputRefFile == RESULT_2DCDP_OK) )
  {
    auWReturn = RESULT_2DCDP_OK;
  }
  else
  {
    auWReturn = RESULT_2DCDP_NG;
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uWJugdeErrorRetry
 *  Description     ：This function, the judgment of the error retry.
 *  Argument        ：m_uBRetryCounter
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWJugdeErrorRetry( void )
{
  uWord_Type auWReturn = ERROR_RETRY_NG;

  if( m_uBRetryCounter > ERROR_RETRY_JUGE )
  {
    m_uBRetryCounter = m_uBRetryCounter - 1;
    auWReturn = ERROR_RETRY_OK;
  }
  else
  {
    m_uBRetryCounter = ERROR_RETRY_INIT;
    auWReturn = ERROR_RETRY_NG;
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：uWCheck2dcdpParameter
 *  Description     ：This function checks the configuration parameters of 2DCDP.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWCheck2dcdpParameter( void )
{
  std::string sfilename      = "";
  Char_Type aBTagName[32]    = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  uByte_Type auBColor_red, auBColor_green, auBColor_blue = 0;
  uByte_Type auBbgOut_red, auBbgOut_green, auBbgOut_blue = 0;
  int asiShrUp = 0;
  int asiShrDn = 0;
  std::string a_2dcdp_opt[8];

  uWord_Type auWReturn = OPTION_DATA_EMPTY;

  if( m_2dcdp_optIn.isNew() == true )
  {
    m_2dcdp_optIn.read();

    sfilename = m_2dcdp_opt.data[OPT_DATA_INDEX_1];
    strcpy( aBTagName, sfilename.c_str() );
    nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_INP ), aBTagName, RTC_2DCDP_IMG_INP );
    nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_REF ), aBTagName, RTC_2DCDP_IMG_REF );
    nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_LBL ), aBTagName, RTC_2DCDP_IMG_LBL );

    nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_INO ), aBTagName, RTC_2DCDP_IMG_INO );
    nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_PRO ), aBTagName, RTC_2DCDP_IMG_PRO );
    nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_RFO ), aBTagName, RTC_2DCDP_IMG_RFO );
    nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_LBO ), aBTagName, RTC_2DCDP_IMG_LBO );
    nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_DIF ), aBTagName, RTC_2DCDP_IMG_DIF );

    a_2dcdp_opt[0] = m_2dcdp_opt.data[OPT_DATA_INDEX_3];
    a_2dcdp_opt[1] = m_2dcdp_opt.data[OPT_DATA_INDEX_4];
    asiShrUp = static_cast<int>(std::stoi( a_2dcdp_opt[0] ));
    asiShrDn = static_cast<int>(std::stoi( a_2dcdp_opt[1] ));

    a_2dcdp_opt[2] = m_2dcdp_opt.data[OPT_DATA_INDEX_5];
    a_2dcdp_opt[3] = m_2dcdp_opt.data[OPT_DATA_INDEX_6];
    a_2dcdp_opt[4] = m_2dcdp_opt.data[OPT_DATA_INDEX_7];
    auBColor_red   = static_cast<uByte_Type>(std::stoul( a_2dcdp_opt[2]) );
    auBColor_green = static_cast<uByte_Type>(std::stoul( a_2dcdp_opt[3]) );
    auBColor_blue  = static_cast<uByte_Type>(std::stoul( a_2dcdp_opt[4]) );

    a_2dcdp_opt[5] = m_2dcdp_opt.data[OPT_DATA_INDEX_8];
    a_2dcdp_opt[6] = m_2dcdp_opt.data[OPT_DATA_INDEX_9];
    a_2dcdp_opt[7] = m_2dcdp_opt.data[OPT_DATA_INDEX_10];
    auBbgOut_red   = static_cast<uByte_Type>(std::stoul( a_2dcdp_opt[5]) );
    auBbgOut_green = static_cast<uByte_Type>(std::stoul( a_2dcdp_opt[6]) );
    auBbgOut_blue  = static_cast<uByte_Type>(std::stoul( a_2dcdp_opt[7]) );

    if( bCheckCoreLogic() )
    {
      nvdSetFileName( GetFileName_p( RTC_2DCDP_TXT_CDP ), aBTagName, RTC_2DCDP_TXT_CDP );
      m_core2dcdp->nvdSetShrUpDn( asiShrUp, asiShrDn );
      m_core2dcdp->nvdSetExtNum( DEFAULT_EXTNUM );
      m_core2dcdp->nvdSetbgColor( auBColor_red, auBColor_green, auBColor_blue );
      m_core2dcdp->nvdSetbgOut( auBbgOut_red, auBbgOut_green, auBbgOut_blue );

      auWReturn = OPTION_DATA_EXIST;
    }
    else
    {
      auWReturn = OPTION_DATA_EMPTY;
    }
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：nvdSetDefaultParameter
 *  Description     ：This function sets the initial data of 2DCDP parameter settings.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvdSetDefaultParameter( void )
{
  Char_Type aBTagName[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  std::string sfilename = "";

  sfilename = "00000";
  strcpy( aBTagName, sfilename.c_str() );
  nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_INP ), aBTagName, RTC_2DCDP_IMG_INP );
  nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_REF ), aBTagName, RTC_2DCDP_IMG_REF );
  nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_LBL ), aBTagName, RTC_2DCDP_IMG_LBL );
  nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_INO ), aBTagName, RTC_2DCDP_IMG_INO );
  nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_PRO ), aBTagName, RTC_2DCDP_IMG_PRO );
  nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_RFO ), aBTagName, RTC_2DCDP_IMG_RFO );
  nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_LBO ), aBTagName, RTC_2DCDP_IMG_LBO );
  nvdSetFileName( GetFileName_p( RTC_2DCDP_IMG_DIF ), aBTagName, RTC_2DCDP_IMG_DIF );

  if( bCheckCoreLogic() )
  {
    nvdSetFileName( GetFileName_p( RTC_2DCDP_TXT_CDP ), aBTagName, RTC_2DCDP_TXT_CDP );
    m_core2dcdp->nvdSetShrUpDn( SHR_UP_MIN, SHR_DN_MIN );
    m_core2dcdp->nvdSetExtNum( DEFAULT_EXTNUM );
    m_core2dcdp->nvdSetbgColor( 255, 0, 0 );
    m_core2dcdp->nvdSetbgOut( 128, 128, 128 );
  }
}

/***************************************************************************************************
 *  Function Name   ：uWCheck2dcdpInputImage
 *  Description     ：This function checks the inputs image data to the 2DCDP RTC.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWCheck2dcdpInputImage( void )
{
  uWord_Type auWReturn   = RESULT_2DCDP_CONTINUE;
  uWord_Type auWRetPort1 = _2DCDP_IMAGE_EMPTY;
  uWord_Type auWRetPort2 = _2DCDP_IMAGE_EMPTY;

  if( (m_uBResultPort1 == _2DCDP_IMAGE_EMPTY) && (m_uBResultPort2 == _2DCDP_IMAGE_EMPTY) )
  {
    auWRetPort1 = uWCheck2dcdpInputImagePort1();
    auWRetPort2 = uWCheck2dcdpInputImagePort2();

    if( (m_uBResultPort1 == _2DCDP_IMAGE_EXIST) && (m_uBResultPort2 == _2DCDP_IMAGE_EXIST) )
    {
      m_Image2dcdpTriggerCons->ReqImageLoaderTrigger( IMLD_TRIG_STOP );
      auWReturn = RESULT_2DCDP_OK;
    }
    else
    {
      auWReturn = uWCheckErrorInputImage( auWRetPort1, auWRetPort2, INP_IMAGE_RES_CHK_BOTH );
    }

  }
  else if( (m_uBResultPort1 == _2DCDP_IMAGE_EXIST) && (m_uBResultPort2 == _2DCDP_IMAGE_EMPTY) )
  {
    auWRetPort2 = uWCheck2dcdpInputImagePort2();

    if( m_uBResultPort2 == _2DCDP_IMAGE_EXIST )
    {
      m_Image2dcdpTriggerCons->ReqImageLoaderTrigger( IMLD_TRIG_STOP );
      auWReturn = RESULT_2DCDP_OK;
    }
    else
    {
      auWRetPort1 = auWRetPort2;
      auWReturn = uWCheckErrorInputImage( auWRetPort1, auWRetPort2, INP_IMAGE_RES_CHK_PORT2 );
    }
  }
  else if( (m_uBResultPort1 == _2DCDP_IMAGE_EMPTY) && (m_uBResultPort2 == _2DCDP_IMAGE_EXIST) )
  {
    auWRetPort1 = uWCheck2dcdpInputImagePort1();

    if( m_uBResultPort1 == _2DCDP_IMAGE_EXIST )
    {
      m_Image2dcdpTriggerCons->ReqImageLoaderTrigger( IMLD_TRIG_STOP );
      auWReturn = RESULT_2DCDP_OK;
    }
    else
    {
      auWRetPort2 = auWRetPort1;
      auWReturn = uWCheckErrorInputImage( auWRetPort1, auWRetPort2, INP_IMAGE_RES_CHK_PORT1 );
    }
  }
  else if( (m_uBResultPort1 == _2DCDP_IMAGE_EXIST) && (m_uBResultPort2 == _2DCDP_IMAGE_EXIST) )
  {
    m_Image2dcdpTriggerCons->ReqImageLoaderTrigger( IMLD_TRIG_STOP );
    auWReturn = RESULT_2DCDP_OK;
  }
  else
  {
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWCheck2dcdpInputImagePort1
 *  Description     ：This function checks the inputs image data to the 2DCDP RTC.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWCheck2dcdpInputImagePort1( void )
{
  uWord_Type auWReturn       = _2DCDP_IMAGE_EMPTY;
  uWord_Type auWResizeResult = RESULT_2DCDP_NG;

  if( m_image1In.isNew() == true )
  {
    m_image1In.read();
    if( ((m_image1.width > XIMG_MIN) && (m_image1.width <= XIMG_MAX)) && ((m_image1.height > YIMG_MIN) && (m_image1.height <= YIMG_MAX)) )
    {
      if( m_image1.bpp == FORMAT_COLOR24 )
      {
        auWResizeResult = uWResizeInputImage( &m_image1, GetFileName_p( RTC_2DCDP_IMG_INP ), _2DCDP_INPUT_FILE );
        if( auWResizeResult == RESULT_2DCDP_OK )
        {
          m_uBResultPort1 = _2DCDP_IMAGE_EXIST;
          auWReturn = _2DCDP_IMAGE_EXIST;
        }
        else
        {
          auWReturn = _2DCDP_IMAGE_RESIZE_ERR;
        }
      }
      else
      {
        std::cout << _2DCDP_ERROR_IMAGE_COLOR << GetFileName_p( RTC_2DCDP_IMG_INP ) << std::endl;
        auWReturn = _2DCDP_IMAGE_COLOR_ERR;
      }
    }
    else
    {
      std::cout << _2DCDP_ERROR_IMAGE_SIZE << GetFileName_p( RTC_2DCDP_IMG_INP ) << std::endl;
      auWReturn = _2DCDP_IMAGE_SIZE_ERR;
    }
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWCheck2dcdpInputImagePort2
 *  Description     ：This function checks the inputs image data to the 2DCDP RTC.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWCheck2dcdpInputImagePort2( void )
{
  uWord_Type auWReturn       = _2DCDP_IMAGE_EMPTY;
  uWord_Type auWResizeResult = RESULT_2DCDP_NG;

  if( m_image2In.isNew() == true )
  {
    m_image2In.read();
    if( ((m_image2.width > XIMG_MIN) && (m_image2.width <= XIMG_MAX)) && ((m_image2.height > YIMG_MIN) && (m_image2.height <= YIMG_MAX)) )
    {
      if( m_image2.bpp == FORMAT_COLOR24 )
      {
        auWResizeResult = uWResizeInputImage( &m_image2, GetFileName_p( RTC_2DCDP_IMG_REF ), _2DCDP_REFERENCE_FILE );
        if( auWResizeResult == RESULT_2DCDP_OK )
        {
          auWResizeResult = uWResizeInputImage( &m_image2, GetFileName_p( RTC_2DCDP_IMG_LBL ), _2DCDP_LABEL_FILE );
          if( auWResizeResult == RESULT_2DCDP_OK )
          {
            m_uBResultPort2 = _2DCDP_IMAGE_EXIST;
            auWReturn = _2DCDP_IMAGE_EXIST;
          }
          else
          {
            auWReturn = _2DCDP_IMAGE_RESIZE_ERR;
          }
        }
        else
        {
          auWReturn = _2DCDP_IMAGE_RESIZE_ERR;
        }
      }
      else
      {
        std::cout << _2DCDP_ERROR_IMAGE_COLOR << GetFileName_p( RTC_2DCDP_IMG_REF ) << std::endl;
        auWReturn = _2DCDP_IMAGE_COLOR_ERR;
      }
    }
    else
    {
      std::cout << _2DCDP_ERROR_IMAGE_SIZE << GetFileName_p( RTC_2DCDP_IMG_REF ) << std::endl;
      auWReturn = _2DCDP_IMAGE_SIZE_ERR;
    }
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWResizeInputImage
 *  Description     ：This function resizes the image data.
 *                  ：Also, it execute writing of image files and reading.
 *  Argument        ：*m_image, *filename, Kind (input file, reference file, label file)
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWResizeInputImage( RTC::CameraImage *am_image, Char_Type *afilename, uByte_Type auBKind )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  cv::Mat m_imagebuf;
  cv::Mat m_resizeimage;

  m_imagebuf = cv::Mat( cv::Size( am_image->width, am_image->height ), CV_8UC3 );
  memcpy( m_imagebuf.data, (void *)&(am_image->pixels[0]), am_image->pixels.length() );

  IplImage_buf = m_imagebuf;
  if( bCheckCoreLogic() )
  {
    if( auBKind == _2DCDP_INPUT_FILE )
    {
      m_core2dcdp->SetInputImage( &IplImage_buf );
      m_resizeimage = cv::cvarrToMat( m_core2dcdp->GetImage_inp() );
      auWReturn = uWWriteInputImage( afilename, m_resizeimage );
    }
    else if( auBKind == _2DCDP_REFERENCE_FILE )
    {
      m_core2dcdp->SetReferenceImage( &IplImage_buf );
      m_resizeimage = cv::cvarrToMat( m_core2dcdp->GetImage_ref() );
      auWReturn = uWWriteInputImage( afilename, m_resizeimage );
    }
    else if( auBKind == _2DCDP_LABEL_FILE )
    {
      m_core2dcdp->SetLabelImage( &IplImage_buf );
      auWReturn = RESULT_2DCDP_OK;
    }
    else
    {
      auWReturn = RESULT_2DCDP_NG;
    }
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uWWriteInputImage
 *  Description     ：This function writes the input image data to the image file.
 *  Argument        ：*filename, cv::Mat dsp_image
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWWriteInputImage( Char_Type *afilename, cv::Mat dsp_image )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;
  Bool_Type  bRet = FALSE;

  if( afilename != NULL )
  {
    if( m_image_write_sw == IMAGE_OK_WRITE )
    {
      bRet = cv::imwrite( afilename, dsp_image );
    }
    else
    {
      bRet = TRUE;
    }

    if( bRet == TRUE )
    {
      auWReturn = RESULT_2DCDP_OK;
    }
    else
    {
      auWReturn = RESULT_2DCDP_NG;
      std::cout << _2DCDP_ERROR_IMAGE_WRITE << afilename << std::endl;
    }
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uWCheckErrorInputImage
 *  Description     ：This function checks for errors of the input image data.
 *  Argument        ：auWResPort1, auWResPort2, auBKind
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWCheckErrorInputImage( uWord_Type auWResPort1, uWord_Type auWResPort2, uByte_Type auBKind )
{
  uWord_Type auWReturn = RESULT_2DCDP_CONTINUE;

  switch ( auBKind )
  {
    case INP_IMAGE_RES_CHK_BOTH:
      if( (auWResPort1 == _2DCDP_IMAGE_RESIZE_ERR) ||
          (auWResPort1 == _2DCDP_IMAGE_COLOR_ERR)  ||
          (auWResPort1 == _2DCDP_IMAGE_SIZE_ERR)   ||
          (auWResPort2 == _2DCDP_IMAGE_RESIZE_ERR) ||
          (auWResPort2 == _2DCDP_IMAGE_COLOR_ERR)  ||
          (auWResPort2 == _2DCDP_IMAGE_SIZE_ERR) )
      {
        auWReturn = RESULT_2DCDP_NG;
      }
      break;

    case INP_IMAGE_RES_CHK_PORT1:
    case INP_IMAGE_RES_CHK_PORT2:
      if( (auWResPort1 == _2DCDP_IMAGE_RESIZE_ERR) ||
          (auWResPort1 == _2DCDP_IMAGE_COLOR_ERR)  ||
          (auWResPort1 == _2DCDP_IMAGE_SIZE_ERR) )
      {
        auWReturn = RESULT_2DCDP_NG;
      }
      break;

    default:
      break;
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uWWriteImage
 *  Description     ：This function writes the image data.
 *  Argument        ：*m_image, *filename, *imgaddress
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWWriteImage( Char_Type *afilename, const IplImage *Ipl_img )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;
  Bool_Type bRet = FALSE;

  cv::Mat Mat_img = cv::cvarrToMat( Ipl_img );

  bRet = FALSE;
  if( m_image_write_sw == IMAGE_OK_WRITE )
  {
    bRet = cv::imwrite( afilename, Mat_img );
  }
  else
  {
    bRet = TRUE;
  }

  if( bRet == TRUE )
  {
    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uWCreationCameraImages
 *  Description     ：This function execute the creation and writing of the multi-image data.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWCreationCameraImages( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  uWord_Type auWResult_Inp = uWSetCameraImages_Inp();
  uWord_Type auWResult_Ref = uWSetCameraImages_Ref();
  uWord_Type auWResult_Pro = uWSetCameraImages_Pro();
  uWord_Type auWResult_Ino = uWSetCameraImages_Ino();
  uWord_Type auWResult_Rfo = uWSetCameraImages_Rfo();
  uWord_Type auWResult_Def = uWSetCameraImages_Def();

  if( (auWResult_Inp == RESULT_2DCDP_OK) && (auWResult_Ref == RESULT_2DCDP_OK) && (auWResult_Ino == RESULT_2DCDP_OK) &&
      (auWResult_Pro == RESULT_2DCDP_OK) && (auWResult_Rfo == RESULT_2DCDP_OK) && (auWResult_Def == RESULT_2DCDP_OK) )
  {
    m_Image_2dcdpOut.write();	
    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImages_Inp
 *  Description     ：This function execute the creation and writing of the multi-image data for input.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImages_Inp( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( m_image_inp_sw == CONFIG_ENABLE_SW )
  {
    #ifdef GET_IMAGE_FROM_IMAGEFILE
    auWReturn = uWSetCameraImages( GetFileName_p( RTC_2DCDP_IMG_INP ), RTC_CAMERA_IMAGE_INP );
    #else
    auWReturn = uWSetCameraImagesbyIplImage( RTC_CAMERA_IMAGE_INP );
    #endif
  }
  else
  {
    nvdSetDefaultCameraImages( RTC_CAMERA_IMAGE_INP );
    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImages_Ref
 *  Description     ：This function execute the creation and writing of the multi-image data for reference.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImages_Ref( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( m_image_ref_sw == CONFIG_ENABLE_SW )
  {
    #ifdef GET_IMAGE_FROM_IMAGEFILE
    auWReturn = uWSetCameraImages( GetFileName_p( RTC_2DCDP_IMG_REF ), RTC_CAMERA_IMAGE_REF );
    #else
    auWReturn = uWSetCameraImagesbyIplImage( RTC_CAMERA_IMAGE_REF );
    #endif
  }
  else
  {
    nvdSetDefaultCameraImages( RTC_CAMERA_IMAGE_REF );
    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImages_Pro
 *  Description     ：This function execute the creation and writing of the multi-image data for input projected.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImages_Pro( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( m_image_op_sw == CONFIG_ENABLE_SW )
  {
    #ifdef GET_IMAGE_FROM_IMAGEFILE
    auWReturn = uWSetCameraImages( GetFileName_p( RTC_2DCDP_IMG_PRO ), RTC_CAMERA_IMAGE_PRO );
    #else
    auWReturn = uWSetCameraImagesbyIplImage( RTC_CAMERA_IMAGE_PRO );
    #endif
  }
  else
  {
    nvdSetDefaultCameraImages( RTC_CAMERA_IMAGE_PRO );
    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImages_Ino
 *  Description     ：This function execute the creation and writing of the multi-image data for database.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImages_Ino( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( m_image_oi_sw == CONFIG_ENABLE_SW )
  {
    #ifdef GET_IMAGE_FROM_IMAGEFILE
    auWReturn = uWSetCameraImages( GetFileName_p( RTC_2DCDP_IMG_INO ), RTC_CAMERA_IMAGE_INO );
    #else
    auWReturn = uWSetCameraImagesbyIplImage( RTC_CAMERA_IMAGE_INO );
    #endif
  }
  else
  {
    nvdSetDefaultCameraImages( RTC_CAMERA_IMAGE_INO );
    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImages_Rfo
 *  Description     ：This function execute the creation and writing of the multi-image data for Query.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImages_Rfo( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( m_image_or_sw == CONFIG_ENABLE_SW )
  {
    #ifdef GET_IMAGE_FROM_IMAGEFILE
    auWReturn = uWSetCameraImages( GetFileName_p( RTC_2DCDP_IMG_RFO ), RTC_CAMERA_IMAGE_RFO );
    #else
    auWReturn = uWSetCameraImagesbyIplImage( RTC_CAMERA_IMAGE_RFO );
    #endif
  }
  else
  {
    nvdSetDefaultCameraImages( RTC_CAMERA_IMAGE_RFO );
    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImages_Def
 *  Description     ：This function execute the creation and writing of the multi-image data for difference.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImages_Def( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( m_image_dif_sw == CONFIG_ENABLE_SW )
  {
    #ifdef GET_IMAGE_FROM_IMAGEFILE
    auWReturn = uWSetCameraImages( GetFileName_p( RTC_2DCDP_IMG_DIF ), RTC_CAMERA_IMAGE_DIF );
    #else
    auWReturn = uWSetCameraImagesbyIplImage( RTC_CAMERA_IMAGE_DIF );
    #endif
  }
  else
  {
    nvdSetDefaultCameraImages( RTC_CAMERA_IMAGE_DIF );
    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWSetDefaultCameraImages
 *  Description     ：This function sets the default data of the Multi-CameraImage type.
 *  Argument        ：auWKind
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvdSetDefaultCameraImages( uWord_Type auWKind )
{
  m_Image_2dcdp.images[auWKind].width  = 0;
  m_Image_2dcdp.images[auWKind].height = 0;
  m_Image_2dcdp.images[auWKind].bpp    = 0;
  m_Image_2dcdp.images[auWKind].format = "";
  m_Image_2dcdp.images[auWKind].fDiv   = 0;
  m_Image_2dcdp.images[auWKind].pixels.length( 8 );
}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImages
 *  Description     ：This function sets the data of the Multi-CameraImage type.
 *  Argument        ：*afilename, auWKind
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImages( Char_Type *afilename, uWord_Type auWKind )
{
  cv::Mat dsp_image;

  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( afilename != NULL )
  {
    dsp_image = cv::imread( afilename, cv::IMREAD_UNCHANGED );
    if ( dsp_image.empty() )
    {
      std::cout << _2DCDP_ERROR_IMAGE_EMPTY << afilename << std::endl;
    }
    else
    {
      if( (auWKind >= RTC_CAMERA_IMAGE_MIN) && (auWKind < RTC_CAMERA_IMAGE_MAX) )
      {
        auWReturn = uWSetCameraImagesData( dsp_image, auWKind );
      }
    }
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImagesbyIplImage
 *  Description     ：This function sets the data of the Multi-CameraImage type.
 *                  ：In addition, this function is used when there is no image file.
 *  Argument        ：auWKind
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImagesbyIplImage( uWord_Type auWKind )
{
  cv::Mat dsp_image;

  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( bCheckCoreLogic() )
  {
    if( (auWKind >= RTC_CAMERA_IMAGE_MIN) && (auWKind < RTC_CAMERA_IMAGE_MAX) )
    {
      switch( auWKind )
      {
        case RTC_CAMERA_IMAGE_INP:
          dsp_image = cv::cvarrToMat( m_core2dcdp->GetImage_inp() );
          auWReturn = uWSetCameraImagesData( dsp_image, RTC_CAMERA_IMAGE_INP );
          break;

        case RTC_CAMERA_IMAGE_REF:
          dsp_image = cv::cvarrToMat( m_core2dcdp->GetImage_ref() );
          auWReturn = uWSetCameraImagesData( dsp_image, RTC_CAMERA_IMAGE_REF );
          break;

        case RTC_CAMERA_IMAGE_INO:
          dsp_image = cv::cvarrToMat( m_core2dcdp->GetImage_o_i() );
          auWReturn = uWSetCameraImagesData( dsp_image, RTC_CAMERA_IMAGE_INO );
          break;

        case RTC_CAMERA_IMAGE_PRO:
          dsp_image = cv::cvarrToMat( m_core2dcdp->GetImage_o_p() );
          auWReturn = uWSetCameraImagesData( dsp_image, RTC_CAMERA_IMAGE_PRO );
          break;

        case RTC_CAMERA_IMAGE_RFO:
          dsp_image = cv::cvarrToMat( m_core2dcdp->GetImage_o_r() );
          auWReturn = uWSetCameraImagesData( dsp_image, RTC_CAMERA_IMAGE_RFO );
          break;

        case RTC_CAMERA_IMAGE_DIF:
          dsp_image = cv::cvarrToMat( m_core2dcdp->GetImage_dif() );
          auWReturn = uWSetCameraImagesData( dsp_image, RTC_CAMERA_IMAGE_DIF );
          break;

        default:
          break;
      }
    }
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：uWSetCameraImagesData
 *  Description     ：This function sets the data of the Multi-CameraImage type.
 *  Argument        ：Mat type dsp_image, auWKind
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWSetCameraImagesData( cv::Mat dsp_image, uWord_Type auWKind )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;

  if( dsp_image.empty() )
  {
    std::cout << _2DCDP_ERROR_IMAGE_EMPTY << auWKind << std::endl;
  }
  else
  {
    sWord_Type asWlength = ( ( dsp_image.channels() * dsp_image.cols ) * dsp_image.rows );
    m_Image_2dcdp.images[auWKind].width  = dsp_image.cols;
    m_Image_2dcdp.images[auWKind].height = dsp_image.rows;
    m_Image_2dcdp.images[auWKind].bpp    = FORMAT_COLOR24;
    m_Image_2dcdp.images[auWKind].format = "bitmap";
    m_Image_2dcdp.images[auWKind].fDiv   = 0;
    m_Image_2dcdp.images[auWKind].pixels.length( asWlength );
    ::memcpy( ( void * )&( m_Image_2dcdp.images[ auWKind ].pixels[ 0 ] ) , dsp_image.data , asWlength );

    auWReturn = RESULT_2DCDP_OK;
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：uWCreationInputRefFile
 *  Description     ：This function execute the path information creation and writing of Input
 *                  ：reference correspondence file.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWCreationInputRefFile( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_NG;
  uWord_Type auWRet    = 0;
  std::string a_2dcdp_path;
  Char_Type buffer[BUFFER_SIZE_1024];

  if( bCheckCoreLogic() )
  {
    auWRet = GetFullPathName( GetFileName_p( RTC_2DCDP_TXT_CDP ), BUFFER_SIZE_1024, buffer, NULL );
  }

  if( auWRet != 0 )
  {
    a_2dcdp_path = std::string( buffer );
    m_Text_2dcdp.data = CORBA::string_dup( a_2dcdp_path.c_str() );
    if( m_file_cdp_sw == CONFIG_ENABLE_SW )
    {
      m_Text_2dcdpOut.write();
    }
    auWReturn = RESULT_2DCDP_OK;
  }
  else
  {
    if( m_file_cdp_sw == CONFIG_DISABLE_SW )
    {
      auWReturn = RESULT_2DCDP_OK;
    }
  }

  return auWReturn;
}

/***************************************************************************************************
 *  Function Name   ：nvdSet2dcdpStageControl
 *  Description     ：This function sets the member variable of m_uB2dcdpStageCtrl.
 *  Argument        ：m_uB2dcdpStageCtrl
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvdSet2dcdpStageControl( uByte_Type auB2dcdpStageCtrl )
{
  if( (auB2dcdpStageCtrl >= _2DCDP_STAGE_MIN) && (auB2dcdpStageCtrl < _2DCDP_STAGE_MAX) )
  {
    m_uB2dcdpStageCtrl = auB2dcdpStageCtrl;
  }
  else
  {
    m_uB2dcdpStageCtrl = _2DCDP_STAGE1;
  }
}

/***************************************************************************************************
 *  Function Name   ：nvd2dcdpOnInitialize
 *  Description     ：When onInitialize, this function execute the initialization.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvd2dcdpOnInitialize( void )
{
  std::cout << _2DCDP_ON_INITIALIZE << std::endl;

  nvdSet2dcdpStageControl( _2DCDP_STAGE_INIT );
}

/***************************************************************************************************
 *  Function Name   ：nvd2dcdpOnActivated
 *  Description     ：When onActivated, this function execute the initialization.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvd2dcdpOnActivated( void )
{
  m_core2dcdp = new CDP( XIMG_120, YIMG_90, XIMG_120, YIMG_90, SHR_UP_MIN, SHR_DN_MIN, CDP::Color(255,0,0) );

  if( bCheckCoreLogic() )
  {
    std::cout << _2DCDP_ON_ACTIVATED << std::endl;

    m_Image_2dcdp.images.length( IMAGES_LENGTH_6 );
    m_2dcdp_opt.data.length( OPT_DATA_LENGTH_11 );
    nvdSet2dcdpStageControl( _2DCDP_STAGE1 );
    nvdSetDefaultParameter();
  }
}

/***************************************************************************************************
 *  Function Name   ：nvd2dcdpOnDeactivated
 *  Description     ：When onDeactivated, this function execute the initialization.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvd2dcdpOnDeactivated( void )
{
  std::cout << _2DCDP_ON_DEACTIVATED << std::endl;

  nvdSet2dcdpStageControl( _2DCDP_STAGE_INIT );
  nvdDeleteCore2dcdp();
}

/***************************************************************************************************
 *  Function Name   ：nvd2dcdpOnAborting
 *  Description     ：When onAborting, this function execute the initialization.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvd2dcdpOnAborting( void )
{
  std::cout << _2DCDP_ON_ABORTING << std::endl;

  m_Image2dcdpTriggerCons->ReqImageLoaderTrigger( IMLD_TRIG_STOP );
  m_uBErrorCounter = 1;
  nvdDeleteCore2dcdp();
}

/***************************************************************************************************
 *  Function Name   ：nvd2dcdpOnError
 *  Description     ：When onError, this function execute the initialization.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvd2dcdpOnError( void )
{
  if( m_uBErrorCounter > 0 )
  {
    std::cout << _2DCDP_ON_ERROR << std::endl;
    m_uBErrorCounter = m_uBErrorCounter - 1;
  }
}

/***************************************************************************************************
 *  Function Name   ：nvd2dcdpOnReset
 *  Description     ：When onReset, this function execute the initialization.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvd2dcdpOnReset( void )
{
  std::cout << _2DCDP_ON_RESET << std::endl;
  nvdDeleteCore2dcdp();
}

/***************************************************************************************************
 *  Function Name   ：nvdSetFileName
 *  Description     ：This function sets the file name to the variable of 2DCDP.
 *  Argument        ：aFileName, aTag, auWKind
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvdSetFileName( Char_Type *aFileName, Char_Type *aTag, uWord_Type auWKind )
{
  if( (aFileName != NULL) && (aTag != NULL) )
  {
    switch( auWKind )
    {
      case RTC_2DCDP_IMG_INP:
      { sprintf( aFileName, "%s_inp.bmp", aTag ); break; }

      case RTC_2DCDP_IMG_REF:
      { sprintf( aFileName, "%s_ref.bmp", aTag ); break; }

      case RTC_2DCDP_IMG_LBL:
      { sprintf( aFileName, "%s_lbl.bmp", aTag ); break; }

      case RTC_2DCDP_IMG_INO:
      { sprintf( aFileName, "%si.bmp", aTag );    break; }

      case RTC_2DCDP_IMG_PRO:
      { sprintf( aFileName, "%sp.bmp", aTag );    break; }

      case RTC_2DCDP_IMG_RFO:
      { sprintf( aFileName, "%so.bmp", aTag );    break; }

	    case RTC_2DCDP_IMG_LBO:
      { sprintf( aFileName, "%sl.bmp", aTag );    break; }

      case RTC_2DCDP_IMG_DIF:
      { sprintf( aFileName, "%sd.bmp", aTag );    break; }

      case RTC_2DCDP_TXT_CDP:
      { sprintf( aFileName, "%s_cdp.txt", aTag ); break; }

      default:
        break;
    }
  }
}

/***************************************************************************************************
 *  Function Name   ：GetFileName_p
 *  Description     ：This function returns a pointer of image file name variable.
 *  Argument        ：auWKind
 *  Return          ：pointer of image file name variable
 **************************************************************************************************/
Char_Type *rtc_2dcdp::GetFileName_p( uWord_Type auWKind )
{
  Char_Type *aCFileName = DummyName;

  switch( auWKind )
  {
    case RTC_2DCDP_IMG_INP:
    { return inpfile;  break; }

    case RTC_2DCDP_IMG_REF:
    { return reffile;  break; }

    case RTC_2DCDP_IMG_LBL:
    { return lblfile;  break; }

    case RTC_2DCDP_IMG_INO:
    { return inpout;   break; }

    case RTC_2DCDP_IMG_PRO:
    { return projout;  break; }

    case RTC_2DCDP_IMG_RFO:
    { return refout;   break; }

	  case RTC_2DCDP_IMG_LBO:
    { return lblout;   break; }

    case RTC_2DCDP_IMG_DIF:
    { return diffdist; break; }

    case RTC_2DCDP_TXT_CDP:
    {
      if( bCheckCoreLogic() )
      {
        return ( m_core2dcdp->Get_taiouFile() );
      }
      break;
    }
    default:
      break;
  }

	return aCFileName;
}

/***************************************************************************************************
 *  Function Name   ：bCheckCoreLogic
 *  Description     ：This function to check the generated state of the core logic software.
 *  Argument        ：nothing special
 *  Return          ：Generation state of the core logic.
 **************************************************************************************************/
Bool_Type rtc_2dcdp::bCheckCoreLogic( void )
{
  Bool_Type bRet = FALSE;

  if( m_core2dcdp != NULL )
  {
    bRet = TRUE;
  }

	return bRet;
}

/***************************************************************************************************
 *  Function Name   ：nvdDeleteCore2dcdp
 *  Description     ：This function delete the generated state of the core logic software.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvdDeleteCore2dcdp( void )
{
  if ( bCheckCoreLogic() )
  {
	  delete m_core2dcdp;
  }
}

/***************************************************************************************************
 *  Function Name   ：nvdExecutionCycleChange
 *  Description     ：This function changes the execution cycle of the RTC.
 *  Argument        ：aDTime : execution cycle time
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvdExecutionCycleChange( Double_Type aDTime )
{
  RTC::ExecutionContextList_var eclst;
  eclst = this->get_owned_contexts();
  eclst[(CORBA::ULong)0]->set_rate( aDTime );
}


/***************************************************************************************************
 *  Function Name   ：uWGuardInputPath
 *  Description     ：This function to monitor the input image-file path.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type rtc_2dcdp::uWGuardInputPath( void )
{
  uWord_Type auWReturn = RESULT_2DCDP_CONTINUE;

  if( m_uWGuardCounter > STAGE2_GUARD_JUDGE )
  {
    m_uWGuardCounter = m_uWGuardCounter - 1;
    if( m_uWGuardCounter == STAGE2_GUARD_JUDGE )
    {
      m_Image2dcdpTriggerCons->ReqImageLoaderTrigger( IMLD_TRIG_STOP );
      auWReturn = RESULT_2DCDP_NG;
    }
    else
    {
      auWReturn = RESULT_2DCDP_CONTINUE;
    }
  }
  else
  {
    m_Image2dcdpTriggerCons->ReqImageLoaderTrigger( IMLD_TRIG_STOP );
    auWReturn = RESULT_2DCDP_NG;
  }

	return auWReturn;
}


#ifdef DEBUG_SW
/***************************************************************************************************
 *  Function Name   ：nvdCycleTimeMeasurement
 *  Description     ：Cycle time measurement of RTC execute.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void rtc_2dcdp::nvdCycleTimeMeasurement( void )
{
  switch( m_cycle_time )
  {
    case 0:
    {
	    rtc_start = GetTickCount();
      m_cycle_time = 1;
      break;
    }

    case 1:
    {
	    rtc_end = GetTickCount();
      std::cout << "Execution cycle time: " << (double)(rtc_end - rtc_start) << std::endl;
      m_cycle_time = 2;
      break;
    }

    case 2:
    {
	    rtc_start = GetTickCount();
      std::cout << "Execution cycle time: " << (double)(rtc_start - rtc_end) << std::endl;
      m_cycle_time = 1;
      break;
    }

    default:
      m_cycle_time = 0;
      break;
  }
}
#endif

//--------------------------------------------------------------------------------------------------

extern "C"
{
 
  void rtc_2dcdpInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_2dcdp_spec);
    manager->registerFactory(profile,
                             RTC::Create<rtc_2dcdp>,
                             RTC::Delete<rtc_2dcdp>);
  }
  
};


